/**
 * 孵化器基础数据相关逻辑
 * 作者: 李某
 * 孵化器相关逻辑 包括孵化器的增删改查
 * 
 */
import moment = require("moment");
import { BaseParamUpdateConfig } from "../../config/checkParamConfig";
import { findEnterpriseCount, findEnterpriseCountByFuHuaQiUscc, groupFindEnterprise } from "../../data/enterprise/enterprise";
import { findFuHuaQiByUSCC, findFuHuaQiCount, findFuHuaQiList, groupFindFuHuaQi } from "../../data/fuHuaQi/fuhuaqi";
import * as tools from "../../util/tools";
import { FuHuaQiBaseListConfig } from "../../config/ojbectResultKeyConfig";
import { FUHUAQILV } from "../../config/enum";


/**
 * 更新孵化器基础数据
 * @param uscc 孵化器统一信用代码
 * @param param 表单内容
 * @returns {isSuccess:true/false};
 */
export async function updateFuHuaQiBaseData(uscc:string, param) {
    let baseDataInfo = await findFuHuaQiByUSCC(uscc);
    
    tools.checkParamater("更新孵化器基础数据", BaseParamUpdateConfig, param);
    
    let changeList = tools.checkChange(param, baseDataInfo);
    for (let i = 0; i < changeList.length; i++) {
        let key = changeList[i];
        baseDataInfo[key] = param[key];
    }

    await baseDataInfo.save();

    return {isSuccess:true};
}


/**
 * 查询孵化器基础数据
 * @param uscc 孵化器统一信用代码
 * @returns data 基础数据字段
 */
export async function selectFuHuaQiBaseData(uscc:string) {
    let dataBaseInfo = await findFuHuaQiByUSCC(uscc);
    let enterpriseTotal = await findEnterpriseCount({fuHuaQiUscc:uscc}); //企业总数
    let numOfEntities = enterpriseTotal - dataBaseInfo.virtualEnterpriseNum; //企业总数 - 虚拟企业数量
    let keyList = [ 
        "name", "operationName", "uscc", "virtualEnterpriseNum", "logonTime", "incubatedAcreage", "acreageTotal", "acreagePersonalUse", 
        "lv", "identificationTime","industry","institutionalNature","liaison","liaisonPhone","personInCharge","personInChargePhone",
        "hatchingGround"];
    let data = {};

    keyList.forEach( key => {
        data[key] = dataBaseInfo[key];
        data["enterpriseTotal"] = enterpriseTotal;
        data["numOfEntities"] = numOfEntities;
    });

    return data;
}

/**
 * 主界面基础数据
 * @param uscc 孵化器统一信用代码
 * @returns data "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "enterpriseCount"
 */
export async function homeData(uscc:string) {
    let dataBaseInfo = await findFuHuaQiByUSCC(uscc);
    let keyList = [ "incubatedAcreage", "acreageTotal", "acreagePersonalUse", "name", "lv", "institutionalNature"];
    let data:any = {enterpriseCount:0};

    keyList.forEach( key => {
        data[key] = dataBaseInfo[key];
    });

    data.enterpriseCount  = await findEnterpriseCountByFuHuaQiUscc(uscc);
    data.lv = FUHUAQILV[dataBaseInfo.lv];
    data.institutionalNature = FUHUAQILV[dataBaseInfo.institutionalNature];
    return data;
}


/**
 * 获取孵化器基本信息列表
 * @param lv 孵化器等级
 * @param institutionalNature 机构性质
 * @param industry 领域
 * @param page 页数
 * @returns 
 */
export async function fuHuaQiBaseList(lv:number, institutionalNature:number, industry:number, page:number) {
    let selectParam:any = {};
    if (lv) selectParam.lv = lv;
    if (institutionalNature) selectParam.institutionalNature = institutionalNature;
    if (industry) selectParam.industry = industry;

    let enterpriseMap = await groupFindEnterprise();
    let dataBaseList = await findFuHuaQiList(selectParam, (page-1)*10 );
    let count = await findFuHuaQiCount(selectParam);
    let dataList = [];
    dataBaseList.forEach( info => {
        let changeData = tools.extractData(FuHuaQiBaseListConfig, info);
        const Count = enterpriseMap[info.uscc] || 0;
        dataList.push(changeData);
    });

    return {count, dataList};
}